@@ -209,6 +209,14 @@ loop:
 			MainInstance.nc.GetLogger().Error("Error updating status file %s: %s", statusFilename, err)
 		}
 	}
+	err = stdin.Close()
+	if err != nil {
+		MainInstance.nc.GetLogger().Error("Error closing %s: %s", path.Join(unitdir, "stdin"), err)
+	}
+	err = stdout.Close()
+	if err != nil {
+		MainInstance.nc.GetLogger().Error("Error closing %s: %s", path.Join(unitdir, "stdout"), err)
+	}
 	os.Exit(cmd.ProcessState.ExitCode())
 
 	return nil
@@ -250,13 +258,13 @@ func (cw *commandUnit) Status() *StatusFileData {
 // UnredactedStatus returns a copy of the status currently loaded in memory, including secrets.
 func (cw *commandUnit) UnredactedStatus() *StatusFileData {
 	cw.GetStatusLock().RLock()
-	defer cw.GetStatusLock().RUnlock()
 	status := cw.GetStatusWithoutExtraData()
 	ed, ok := cw.GetStatusCopy().ExtraData.(*CommandExtraData)
 	if ok {
 		edCopy := *ed
 		status.ExtraData = &edCopy
 	}
+	cw.GetStatusLock().RUnlock()
 
 	return status
 }
